#!/usr/bin/perl
package CSMSIS;

use strict;
use warnings;
use Carp; 
use File::Copy;

use lib ($ENV{'CSM_PM'} || '/opt/csm/pm');
use FindBin qw($Bin);
use lib "$Bin";

use NodeUtils;




										

# Global
$::SISIMAGEDIR = '/csminstall/sis/images';
$::CSMGETSISIMAGE_LOCK = "/tmp/csmgetsisimage.sem";

$::MKZIMAGE = "/bin/mkzimage";

#----------------------------------------------------------------------

=head3  SIS_check_golden_client_status

	Description:
	check the Mode of golden client, make sure it is MinManaged or Managed
	or an error message will be given
	
=cut

#----------------------------------------------------------------------
sub SIS_check_golden_client_status()
{
    my $routine = "SIS_check_golden_client_status";
	print "ENTERING: $routine\n" if $::DEBUG;

    my($class, $golden_client) = @_;
	my($host, $ipaddr) = NetworkUtils->getHost($golden_client);
    my $node_mode = $::DestNodeHash{$host}{"Mode"};

    print "Checking golden client status: $host.\n";
    if ($node_mode eq 'MinManaged' || $node_mode eq 'Managed')
    {
        return $::OK;
    }
    else
    {
        MessageUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,
                'csminstall','E1', 'EMsgStatusSupported', $node_mode);
        return $::NOK;
    }
    print "LEAVING: $routine\n" if $::DEBUG;
}

#----------------------------------------------------------------------

=head3  SIS_prepare_client

	Description:
	run si_prepareclient on golden client via dsh
	
=cut

#----------------------------------------------------------------------

sub SIS_prepare_client
{
    my $routine = "SIS_prepare_client";
	print "ENTERING: $routine\n" if $::DEBUG;

    my($class, $golden_client, $image_server) = @_;

    my %options_api = ();
    $options_api{'nodes'} = $golden_client;
    $options_api{'command'} = "si_prepareclient --server $image_server --quiet --my-modules </dev/null";

    print "Preparing golden client: $golden_client.\nThis may take some time to complete.\n";
	my @results = NodeUtils->runDsh(\%options_api, -1);

	if ( $::RUNCMD_RC != 0)
	{
	    MessageUtils->message('E', 'EMsgNO_DSH');
	}
	chop @results;
	foreach my $line (@results)
	{
	    MessageUtils->message('I', 'IMsgShow_Output', $line);
	}

    # wait si_prepareclient to finish
    sleep 200;

    print "LEAVING: $routine\n" if $::DEBUG;
    return $::OK;
}

#----------------------------------------------------------------------

=head3  SIS_modify_systemimager_conf

	Description:
	set sis configure info to systemimager.conf file
	such as default image dir, etc.
	
=cut

#----------------------------------------------------------------------
sub SIS_modify_systemimager_conf()
{
    my $routine = "SIS_modify_systemimager_conf";
	print "ENTERING: $routine\n" if $::DEBUG;

    print "Updating /etc/systemimager.conf file.\n";
    my ($attribute, $dir);
    unless (open(SISCONF_ORG, "<$::SISCFGFILE"))
    {
        MessageUtils->message('E', 'EMsgCANT_READ_FILE', $::SISCFGFILE);
        return $::NOK;
    }
    unless (open(SISCONF_NEW, ">$::TMPSISCFG"))
    {
        MessageUtils->message('E', 'EMsgCANT_WRITE_FILE', $::TMPSISCFG);
        return $::NOK;
    }

    while (my $line = <SISCONF_ORG>)
    {
        if ($line =~ /^\#/ || $line =~ /^s*$/)
        {
            print SISCONF_NEW $line;
        }
        elsif ( $line =~ /^\s*(\S+)\s*=\s*(\S+)\s*$/)
        {
            ($attribute, $dir) = ( $1, $2);
            if ($attribute eq 'DEFAULT_IMAGE_DIR')
            {
                my $cmd = "mkdir -m 755 -p $::SISIMAGEDIR";
                my $output = NodeUtils->runcmd($cmd, 0);
                $dir =  $::SISIMAGEDIR;
            }
            elsif ($attribute eq 'AUTOINSTALL_BOOT_DIR')
            {
                $dir = $::SISTFTPBOOTDIR;
            }
            elsif ($attribute eq 'NET_BOOT_DEFAULT')
            {
                $dir = 'local';
            }
            print SISCONF_NEW "$attribute = $dir\n";
        }
        else
        {
            print SISCONF_NEW $line;
        }
    }

    close(SISCONF_ORG);
    close(SISCONF_NEW);
    my $cp = "cp $::TMPSISCFG $::SISCFGFILE";
    my $output = NodeUtils->runcmd($cp, 0);

    print "Launching SIS installation daemons\n";
    NodeUtils->runcmd("/etc/init.d/systemimager-server-rsyncd start", 0);

    print "LEAVING: $routine\n" if $::DEBUG;
}

#----------------------------------------------------------------------

=head3  SIS_get_image

	Description:
	get clone image from golden client
	
=cut

#----------------------------------------------------------------------

sub SIS_get_image()
{
    print "Getting image from golden client...\n" if $::DEBUG;
    
    my $routine = "SIS_get_image";
	print "ENTERING: $routine\n" if $::DEBUG;

    my ($class, $golden_client, $img_name) = @_;
   
    print "Getting image $img_name from golden client $golden_client...\n"; 
    my $cmd = 'si_getimage --golden-client ' . $golden_client .
              ' --image ' . $img_name . ' --quiet';

    $cmd .= ' --post-install reboot';
    $cmd .= ' --ip-assignment static' . ' --update-script YES';

    my @result = NodeUtils->runcmd($cmd, -1);
    if ($::RUNCMD_RC)
    {
        MessageUtils->messageFromCat('csmInstall.cat', $::MSGMAPPATH,
                                     'csminstall',     'I',
                                     'IMsgShow_Output', join('\n', @result));
        return $::RUNCMD_RC;
    }
    print "LEAVING: $routine\n" if $::DEBUG;
    return $::OK;
}

#----------------------------------------------------------------------

=head3  SIS_record_image_info

	Description:
	record the golden client related information in file 
	imgdir/golden_client.csm. And this info will be used to
	compared with the target node attributes.
	
=cut

#----------------------------------------------------------------------

sub SIS_record_image_info()
{
    my $routine = "SIS_record_image_info";
	print "ENTERING: $routine\n" if $::DEBUG;

    my ($class, $image_dir, $golden_client) = @_;

    my $file = $image_dir . "/golden_client.csm";
    local *FILE;
    open(FILE,">$file")
        || MessageUtils->messageFromCat(
                'csmInstall.cat',     $::MSGMAPPATH,
                'csminstall',         'E2',
                'EMsgCANT_WRITE_FILE', $file
                );

    my $install_csm_version = $::DestNodeHash{$golden_client}{"InstallCSMVersion"};
    my $install_distro_name = $::DestNodeHash{$golden_client}{"InstallDistributionName"};
    my $install_dirtro_version = $::DestNodeHash{$golden_client}{"InstallDistributionVersion"};
    my $install_os_name = $::DestNodeHash{$golden_client}{"InstallOSName"};
    my $install_pkg_arch = $::DestNodeHash{$golden_client}{"InstallPkgArchitecture"};
    my $install_service_level = $::DestNodeHash{$golden_client}{"InstallServiceLevel"};
    my $mode = $::DestNodeHash{$golden_client}{"Mode"};

    my $content = "GoldenClientName = $golden_client\n";
$content = "InstallCSMVersion = $install_csm_version\n";
    $content .= "InstallDistributionName = $install_distro_name\n";
    $content .= "InstallDistributionVersion = $install_dirtro_version\n";
    $content .= "InstallOSName = $install_os_name\n";
    $content .= "InstallPkgArchitecture = $install_pkg_arch\n";
    $content .= "InstallServiceLevel = $install_service_level\n";
    $content .= "Mode = $mode\n";
    
    print FILE $content;
    close(FILE);
    print "LEAVING: $routine\n" if $::DEBUG;
}

#----------------------------------------------------------------------

=head3  SIS_shutdonw_rsyncd

	Description:
	shut down the rsyncd daemon on golden client for security consideration
    furthermore, it is necessary when golden client is the same machine
    with image server, otherwise, systemimager-server-rsyncd can not be 
    start successfully.
	
=cut

#----------------------------------------------------------------------

sub SIS_shutdonw_rsyncd()
{
    my $routine = "SIS_shutdonw_rsyncd";
	print "ENTERING: $routine\n" if $::DEBUG;

    my ($class, $golden_client) = @_;
    my %options_api = ();

    $options_api{'command'} = "killall -HUP rsync";
    $options_api{'nodes'} = $golden_client;
    my @local_dsh_results = NodeUtils->runDsh(\%options_api, -1);

    chop @local_dsh_results;
    if ( $::RUNCMD_RC != 0)
    {
        MessageUtils->message('E', 'EMsgNO_DSH');
    } 
    foreach my $line (@local_dsh_results)
    {
        MessageUtils->message('I', 'IMsgShow_Output', $line);
    }

    print "LEAVING: $routine\n" if $::DEBUG;
    return;
}

sub SIS_copy_ppc64_zimage()
{
    my ($class, $nodelist) = @_;
    my @targetNodes = @$nodelist;

    my @arch_img = CSMSIS->SIS_get_arch_and_img(\@targetNodes);
    my %conf_para = NodesetUtils->get_SIS_conf_para();
    my $img__dir = $conf_para{'default_image_dir'};

    foreach my $item (@arch_img)
    {
        my ($arch, $img) = split ('-', $item);
        my $csm_zimage = $::TFTPBOOT . "/csm/" . $arch . "-" . $img . "-zImage";
        my $sis_dir = "$img__dir/$img/etc/systemimager/boot"; 
        my $sis_zimage = "$sis_dir/zImage";
        
        NodesetUtils::mkpath("$::TFTPBOOT/csm", $::VERBOSE, 0110);
        NetworkUtils->setDefaultFilePermission("$::TFTPBOOT/csm", 1);

	NodesetUtils->copyfile($sis_zimage, $csm_zimage);
	NetworkUtils->setDefaultFilePermission("$csm_zimage");
    }
    return $::OK;
}

sub SIS_build_ppc64_zimage()
{
    my ($class, $distro_name, $distro_version, $golden_client) = @_;

    my $sis_dir = "/etc/systemimager/boot";
    my $kernel = "$sis_dir/kernel";
    my $initrd = "$sis_dir/initrd.img";
    
    if ($distro_name =~ /RedHatEL/)
    {
        # the usage of mkzimage is different between RedHat and SuSE
        my $mkzimage = "/usr/bin/mkzimage";
        $::LS = '/bin/ls';
        my %options_api = ();
        $options_api{'nodes'} = $golden_client;
        # find zImage.stub file on golden client
	if ($distro_version eq "3")
	{
   	    # command to locate the stub file to rebuild ppc zimage
    	    $options_api{'command'} = "$::LS /boot/vmlinuz-partial*";
	}
	elsif ($distro_version eq "4" || $distro_version =~ /4\./)
	{
       	    # command to locate the stub file to rebuild ppc zimage
	    $options_api{'command'} = "$::LS /boot/zImage.stub*";
	}
        elsif ($distro_version eq "5" || $distro_version =~ /5\./)
        {
            $options_api{'command'} = "$::LS /usr/share/ppc64-utils/zImage.stub";
        }

	my @filelist = NodeUtils->runDsh(\%options_api, -1);
	if ($::RUNCMD_RC)
	{
	    MessageUtils->messageFromCat(
	   				 'csmInstall.cat', $::MSGMAPPATH,
	    				 'csminstall',     'I',
	    				 'IMsgShow_Output', join('\n', @filelist)
	    				);
	    return $::RUNCMD_RC;
	}
	my (undef, $zimage_stub) = split (': ', $filelist[0]);

        # /usr/bin/mkzimage kernel no no initrd.img zImage.stub /tmp

	if ($distro_version eq "3")
	{
	    $options_api{'command'} = 
	        "$mkzimage $zimage_stub $sis_dir/zImage $initrd";
	}
	elsif ($distro_version eq "4" || $distro_version =~ /4\./)
	{
	    # mkzimage kernel no no initrd.img /boot/zImage.stub-2.6.9-67.EL zImage1
	    $options_api{'command'} = 
	        "$mkzimage $kernel no no $initrd $zimage_stub $sis_dir/zImage";
	}
        elsif ($distro_version eq "5" || $distro_version =~ /5\./)
        {
	    $options_api{'command'} = 
	        "$mkzimage $kernel no no $initrd $zimage_stub $sis_dir/zImage";
        }

    	my @results = NodeUtils->runDsh(\%options_api, -1);

    	if ( $::RUNCMD_RC != 0)
    	{
    	    MessageUtils->message('E', 'EMsgNO_DSH');
        	chop @results;
        	foreach my $line (@results)
        	{
        	    MessageUtils->message('I', 'IMsgShow_Output', $line);
        	}        
    	}
    }
    elsif ($distro_name =~ /SLES/)
    {
        # the usage of mkzimage is different between RedHat and SuSE
        my $mkzimage = "/bin/mkzimage";

        my %options_api = ();
        $options_api{'nodes'} = $golden_client;

        # mkzimage --vmlinux kernel --initrd initrd.img --output zImage1 --tmp /tmp
        $options_api{'command'} = 
            "$mkzimage --vmlinux $kernel --initrd $initrd " . 
                         "--output $sis_dir/zImage --tmp /tmp";

    	my @results = NodeUtils->runDsh(\%options_api, -1);

    	if ( $::RUNCMD_RC != 0)
    	{
    	    MessageUtils->message('E', 'EMsgNO_DSH');
        	chop @results;
        	foreach my $line (@results)
        	{
        	    MessageUtils->message('I', 'IMsgShow_Output', $line);
        	}        
    	}
    }
    return $::OK;
}

sub SIS_get_arch_and_img()
{
    my ($class, $nodelist) = @_;

    my @arch_img = ();
    
    foreach my $node (@$nodelist)
    {
        my $arch = $::NODEHASH{$node}{InstallPkgArchitecture};
        my $img = $::NODEHASH{$node}{InstallImage};

	if (!$arch || !$img)
	{ next; }
		
	my $found = 1;
	my $key = "$arch" . "-$img";
	
	foreach my $item (@arch_img)
	{
	    if ($item eq $key)
	    {
		$found = 0;
		last;
	    }
	}
	if ($found)
	{
		push @arch_img, $key;
	}
   
    }

    return @arch_img;
}
1;
